use std::marker::PhantomData;

use organic::types::Document;
use serde::Deserialize;
use serde::Serialize;

use super::ast_node::WasmAstNode;
use super::macros::to_wasm;
use super::standard_properties::WasmStandardProperties;
use super::to_wasm::ToWasm;
use crate::wasm::to_wasm::ToWasmStandardProperties;

#[derive(Debug, Serialize, Deserialize)]
#[serde(tag = "ast_node")]
#[serde(rename = "org-data")]
pub(crate) struct WasmDocument<'s> {
    standard_properties: WasmStandardProperties,
    additional_properties: Vec<(String, String)>,
    children: Vec<()>,
    phantom: PhantomData<&'s ()>,
}

to_wasm!(
    WasmDocument<'s>,
    Document<'s>,
    original,
    wasm_context,
    standard_properties,
    {
        let additional_properties: Vec<(String, &str)> = original
            .get_additional_properties()
            .map(|node_property| {
                (
                    format!(":{}", node_property.property_name.to_uppercase()),
                    node_property.value.unwrap_or(""),
                )
            })
            .collect();

        Ok(WasmDocument {
            standard_properties,
            additional_properties: Vec::new(),
            children: Vec::new(),
            phantom: PhantomData,
        })
    }
);

impl<'s> Into<WasmAstNode<'s>> for WasmDocument<'s> {
    fn into(self) -> WasmAstNode<'s> {
        WasmAstNode::Document(self)
    }
}
